# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class PublicHousingOffersTestController < ::Api::D2::ApplicationController
        skip_before_filter :verify_authenticity_token
        skip_before_filter :set_user

        def index
          @query_precision = { published: true }
          aggregation = index_aggregation(target_model: Hermes::HousingOffer, no_acl: true)
          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1

          paginate(
            aggregation,
            each_serializer: Hermes::PublicHousingOfferSerializer,
            per_page: per_page,
            page: page
          )
        end

        def set_instance
          @instance = Hermes::HousingOffer.find_by(published: true, id: params[:id])
        end

        %w[create update destroy].each do |cud_method_name|
          define_method(cud_method_name) do
            render_json_error :unprocessable_entity, :not_implemented
          end
        end

        private

        def index_aggregation(target_model: @model, no_acl: false)
          return unless target_model

          precision = @query_precision || {}
          precision[:user_id] = params[:responsible] if params[:responsible]
          query = target_model
          query = query.readable_by(@user) unless no_acl
          query.where(build_query(target_model).deep_merge(precision))
               .order_by(order_params)
        end

        def order_params
          if %w[
            ActionController::Parameters
            Mongoid::Document
            Hash
          ].include?(params[:sort].class.name)
            params[:sort].each_with_object({}) do |(key, val), h|
              next unless handle_order_param(val)

              h[key] = handle_order_param(val)
            end
          end
        end

        def handle_order_param(val)
          return unless [1, -1, '1', '-1', 'asc', 'desc', :asc, :desc].include?(val)

          val = Integer(val) if ['1', '-1'].include?(val)
          if [1,-1].include? val
            val == 1 ? :asc : :desc
          elsif %w[asc desc].include?(val)
            val.to_sym
          else
            val
          end
        end
      end
    end
  end
end
